#!perl -w
use strict;
use Test::More tests => 22;
use File::Spec;

my $inc = join(' ', map { "-I$_" } @INC);

sub output ($) {
    my $file = File::Spec->catfile( qw( t data 01 ), shift );
    die "$file can not be found!" if ! -e $file;
    return qx($^X $inc -MDevel::LeakTrace::Fast $file 2>&1);
}

is( output '01-no-statement.pl',  '', 'no statements = no leak' );
is( output '02-single-scalar.pl', '', 'single scalar = no leak' );

local $_ = output '03-leak-scalar.pl';
  ok( $_, 'leak a reference loop $foo = \$foo' );
like( $_, qr/^leaked SV\(.*?\) from main line \d+$/m,  'LIKE one SV');
  ok(      s/^leaked SV\(.*?\) from main line \d+$//m, 'one SV');
like( $_, qr/^leaked SV\(.*?\) from main line \d+$/m,  'LIKE another SV');
  ok(      s/^leaked SV\(.*?\) from main line \d+$//m, 'another SV');
  ok(      m/^\n*$/,                                   "and that's all" );

$_ = output '04-leak-array.pl';

  ok( $_, 'leak a reference loop $foo[1] = \@foo' );
like( $_, qr/^leaked SV\(.*?\) from main line \d+$/m,  'LIKE one SV' );
  ok(      s/^leaked SV\(.*?\) from main line \d+$//m, 'one SV' );
like( $_, qr/^leaked AV\(.*?\) from main line \d+$/m,  'LIKE one AV' );
  ok(      s/^leaked AV\(.*?\) from main line \d+$//m, 'one AV' );
like( $_, qr/^leaked RV\(.*?\) from main line \d+$/m,  'LIKE one RV' );
  ok(      s/^leaked RV\(.*?\) from main line \d+$//m, 'one RV' );
  ok(      m/^\n*$/,                                   "and that's all" );

TODO: {
    local $TODO = 'Line numbering is buggy. All of them are line 1';
    local $_    = output '03-leak-scalar.pl';
      ok( $_, 'leak a reference loop $foo = \$foo' );
    like( $_, qr/^leaked SV\(.*?\) from main line 3$/m,  'LIKE one SV' );
      ok(      s/^leaked SV\(.*?\) from main line 3$//m, 'one SV' );
    like( $_, qr/^leaked SV\(.*?\) from main line 3$/m,  'LIKE another SV' );
      ok(      s/^leaked SV\(.*?\) from main line 3$//m, 'another SV' );
      ok(      m/^\n*$/,                                 "and that's all" );
}
